@@ -58,10 +58,10 @@
 #include <qbluetoothservicediscoveryagent.h>
 #include <QDebug>
 #include <QList>
+#include <QMetaEnum>
 #include <QTimer>
 
-Device::Device():
-    connected(false), controller(0), m_deviceScanState(false), randomAddress(false)
+Device::Device()
 {
     //! [les-devicediscovery-1]
     discoveryAgent = new QBluetoothDeviceDiscoveryAgent();
@@ -109,7 +109,7 @@ void Device::startDeviceDiscovery()
 void Device::addDevice(const QBluetoothDeviceInfo &info)
 {
     if (info.coreConfigurations() & QBluetoothDeviceInfo::LowEnergyCoreConfiguration) {
-        DeviceInfo *d = new DeviceInfo(info);
+        auto d = new DeviceInfo(info);
         devices.append(d);
         setUpdate("Last device added: " + d->getName());
     }
@@ -151,9 +151,13 @@ void Device::scanServices(const QString &address)
 {
     // We need the current device for service discovery.
 
-    for (int i = 0; i < devices.size(); i++) {
-        if (((DeviceInfo*)devices.at(i))->getAddress() == address )
-            currentDevice.setDevice(((DeviceInfo*)devices.at(i))->getDevice());
+    for (auto d: qAsConst(devices)) {
+        auto device = qobject_cast<DeviceInfo *>(d);
+        if (!device)
+            continue;
+
+        if (device->getAddress() == address )
+            currentDevice.setDevice(device->getDevice());
     }
 
     if (!currentDevice.getDevice().isValid()) {
@@ -173,13 +177,13 @@ void Device::scanServices(const QString &address)
     if (controller && m_previousAddress != currentDevice.getAddress()) {
         controller->disconnectFromDevice();
         delete controller;
-        controller = 0;
+        controller = nullptr;
     }
 
     //! [les-controller-1]
     if (!controller) {
         // Connecting signals and slots for connecting to LE services.
-        controller = new QLowEnergyController(currentDevice.getDevice());
+        controller = QLowEnergyController::createCentral(currentDevice.getDevice());
         connect(controller, &QLowEnergyController::connected,
                 this, &Device::deviceConnected);
         connect(controller, QOverload<QLowEnergyController::Error>::of(&QLowEnergyController::error),
@@ -211,7 +215,7 @@ void Device::addLowEnergyService(const QBluetoothUuid &serviceUuid)
         return;
     }
     //! [les-service-1]
-    ServiceInfo *serv = new ServiceInfo(service);
+    auto serv = new ServiceInfo(service);
     m_services.append(serv);
 
     emit servicesUpdated();
@@ -228,9 +232,12 @@ void Device::serviceScanDone()
 
 void Device::connectToService(const QString &uuid)
 {
-    QLowEnergyService *service = 0;
-    for (int i = 0; i < m_services.size(); i++) {
-        ServiceInfo *serviceInfo = (ServiceInfo*)m_services.at(i);
+    QLowEnergyService *service = nullptr;
+    for (auto s: qAsConst(m_services)) {
+        auto serviceInfo = qobject_cast<ServiceInfo *>(s);
+        if (!serviceInfo)
+            continue;
+
         if (serviceInfo->getUuid() == uuid) {
             service = serviceInfo->service();
             break;
@@ -256,8 +263,8 @@ void Device::connectToService(const QString &uuid)
 
     //discovery already done
     const QList<QLowEnergyCharacteristic> chars = service->characteristics();
-    foreach (const QLowEnergyCharacteristic &ch, chars) {
-        CharacteristicInfo *cInfo = new CharacteristicInfo(ch);
+    for (const QLowEnergyCharacteristic &ch : chars) {
+        auto cInfo = new CharacteristicInfo(ch);
         m_characteristics.append(cInfo);
     }
 
@@ -279,7 +286,7 @@ void Device::errorReceived(QLowEnergyController::Error /*error*/)
     setUpdate(QString("Back\n(%1)").arg(controller->errorString()));
 }
 
-void Device::setUpdate(QString message)
+void Device::setUpdate(const QString &message)
 {
     m_message = message;
     emit updateChanged();
@@ -318,7 +325,7 @@ void Device::serviceDetailsDiscovered(QLowEnergyService::ServiceState newState)
         return;
     }
 
-    QLowEnergyService *service = qobject_cast<QLowEnergyService *>(sender());
+    auto service = qobject_cast<QLowEnergyService *>(sender());
     if (!service)
         return;
 
@@ -326,8 +333,8 @@ void Device::serviceDetailsDiscovered(QLowEnergyService::ServiceState newState)
 
     //! [les-chars]
     const QList<QLowEnergyCharacteristic> chars = service->characteristics();
-    foreach (const QLowEnergyCharacteristic &ch, chars) {
-        CharacteristicInfo *cInfo = new CharacteristicInfo(ch);
+    for (const QLowEnergyCharacteristic &ch : chars) {
+        auto cInfo = new CharacteristicInfo(ch);
         m_characteristics.append(cInfo);
     }
     //! [les-chars]
@@ -341,8 +348,11 @@ void Device::deviceScanError(QBluetoothDeviceDiscoveryAgent::Error error)
         setUpdate("The Bluetooth adaptor is powered off, power it on before doing discovery.");
     else if (error == QBluetoothDeviceDiscoveryAgent::InputOutputError)
         setUpdate("Writing or reading from the device resulted in an error.");
-    else
-        setUpdate("An unknown error has occurred.");
+    else {
+        static QMetaEnum qme = discoveryAgent->metaObject()->enumerator(
+                    discoveryAgent->metaObject()->indexOfEnumerator("Error"));
+        setUpdate("Error: " + QLatin1String(qme.valueToKey(error)));
+    }
 
     m_deviceScanState = false;
     emit devicesUpdated();
@@ -356,9 +366,7 @@ bool Device::state()
 
 bool Device::hasControllerError() const
 {
-    if (controller && controller->error() != QLowEnergyController::NoError)
-        return true;
-    return false;
+    return (controller && controller->error() != QLowEnergyController::NoError);
 }
 
 bool Device::isRandomAddress() const
